<?php
/** miniOrange enables user to log in using otp credentials.
 * Copyright (C) 2015  miniOrange
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 * @package        miniOrange OAuth
 * @license        http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 */
/**
 * This class contains all the utility functions
 **/
defined('_JEXEC') or die('Restricted access');
jimport('miniorangeotpplugin.utility.commonOtpUtilities');

class MoOtpUtility
{
    public static function getCustomerKeys($isMiniorange=false){
        $keys=array();
        if($isMiniorange){
            $keys['customer_key']= "16555";
            $keys['apiKey']      = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";
        }
        else{
            $columnName = array('customer_key','api_key');
            $details=commonOtpUtilities::getCustomerDetails($columnName, '#__miniorange_otp_customer', 'loadObjectList', array('id' => 1,));
            $keys['customer_key']= $details[0]->customer_key;
            $keys['apiKey']      = $details[0]->api_key;
        }
        return $keys;
    }

    public static function fetchLicense() {
        if(!self::is_curl_installed()) {
            return json_encode(array("status"=>'CURL_ERROR','statusMessage'=>'<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.'));
        }
        $hostname = commonOtpUtilities::getHostname();
        $url = $hostname . '/moas/rest/customer/license';

        $customerKeys= self::getCustomerKeys(false);
        $customerKey = $customerKeys['customer_key'];
        $apiKey      = $customerKeys['apiKey'];
        $fields = array (
            'customerId' => $customerKey,
            'applicationName' => 'otp_recharge_plan'
        );
        $api = new MoOtpUtility();
        return $api->make_curl_call($url,$fields,$api->get_http_header_array()) ;

    }

    function get_http_header_array($isMiniOrange=false) {
        $customerKeys= self::getCustomerKeys($isMiniOrange);
        $customerKey = $customerKeys['customer_key'];
        $apiKey      = $customerKeys['apiKey'];

        /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
       // $currentTimeInMillis = MoTfa_api::get_timestamp();

        /* Creating the Hash using SHA-512 algorithm */
        $stringToHash = $customerKey . $apiKey;;
        $hashValue = hash( "sha512", $stringToHash );

        return array(
            "Content-Type: application/json",
            "Customer-Key: ".$customerKey,
            //"Timestamp: ".$currentTimeInMillis,
            "Authorization: ".$hashValue
        );
    }

    function make_curl_call( $url, $fields, $http_header_array =array( 'Content-Type: application/json', 'charset: UTF - 8', 'Authorization: Basic' ) ) {

        if ( gettype( $fields ) !== 'string' ) {
            $fields = json_encode( $fields );
        }

        return ($this->mo_otp_curl($url, $fields,$http_header_array));
    }

    public function mo_otp_curl($url,$fields,$http_header_array){
        $ch     = curl_init( $url );
        curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
        curl_setopt( $ch, CURLOPT_ENCODING, "" );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );    # required for https urls
        curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
        curl_setopt( $ch, CURLOPT_HTTPHEADER, $http_header_array );
        curl_setopt( $ch, CURLOPT_POST, true);
        curl_setopt( $ch, CURLOPT_POSTFIELDS, $fields);
        $content = curl_exec( $ch );
        if( curl_errno( $ch ) ){
            echo 'Request Error:' . curl_error( $ch );
            exit();
        }
        curl_close( $ch );
        return $content;
    }

    public static function updateLicenseDetails($sample){
        if(!isset($sample->licensePlan))
            return;
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        // Fields to update.
        $fields = array(
            $db->quoteName('license_plan').' ='.$db->quote($sample->licensePlan),
            $db->quoteName('sms_count') . ' = '.$db->quote($sample->smsRemaining),
            $db->quoteName('email_count') . ' = '.$db->quote($sample->emailRemaining),
        );
        // Conditions for which records should be updated.
        $conditions = array(
            $db->quoteName('id') . ' = 1'
        );
        $query->update($db->quoteName('#__miniorange_otp_customer'))->set($fields)->where($conditions);
        $db->setQuery($query);
        $db->execute();

    }

    public static function is_curl_installed()
    {
        if (in_array('curl', get_loaded_extensions())) {
            return 1;
        } else
            return 0;
    }

}